/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.casenorm.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.casenorm.entity.CaseNorm;
import com.jeeplus.modules.casenorm.service.CaseNormService;

/**
 * 案例-造价及材料指标Controller
 * @author fgy
 * @version 2018-01-24
 */
@Controller
@RequestMapping(value = "${adminPath}/casenorm/caseNorm")
public class CaseNormController extends BaseController {

	@Autowired
	private CaseNormService caseNormService;
	
	@ModelAttribute
	public CaseNorm get(@RequestParam(required=false) String id) {
		CaseNorm entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = caseNormService.get(id);
		}
		if (entity == null){
			entity = new CaseNorm();
		}
		return entity;
	}
	
	/**
	 * 案例-造价及材料指标列表页面
	 */
	@RequiresPermissions("casenorm:caseNorm:list")
	@RequestMapping(value = {"list", ""})
	public String list(CaseNorm caseNorm, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CaseNorm> page = caseNormService.findPage(new Page<CaseNorm>(request, response), caseNorm); 
		model.addAttribute("page", page);
		return "modules/casenorm/caseNormList";
	}

	/**
	 * 查看，增加，编辑案例-造价及材料指标表单页面
	 */
	@RequiresPermissions(value={"casenorm:caseNorm:view","casenorm:caseNorm:add","casenorm:caseNorm:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CaseNorm caseNorm, Model model) {
		model.addAttribute("caseNorm", caseNorm);
		return "modules/casenorm/caseNormForm";
	}

	/**
	 * 保存案例-造价及材料指标
	 */
	@RequiresPermissions(value={"casenorm:caseNorm:add","casenorm:caseNorm:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CaseNorm caseNorm, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, caseNorm)){
			return form(caseNorm, model);
		}
		if(!caseNorm.getIsNewRecord()){//编辑表单保存
			CaseNorm t = caseNormService.get(caseNorm.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(caseNorm, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			caseNormService.save(t);//保存
		}else{//新增表单保存
			caseNormService.save(caseNorm);//保存
		}
		addMessage(redirectAttributes, "保存案例-造价及材料指标成功");
		return "redirect:"+Global.getAdminPath()+"/casenorm/caseNorm/?repage";
	}
	
	/**
	 * 删除案例-造价及材料指标
	 */
	@RequiresPermissions("casenorm:caseNorm:del")
	@RequestMapping(value = "delete")
	public String delete(CaseNorm caseNorm, RedirectAttributes redirectAttributes) {
		caseNormService.delete(caseNorm);
		addMessage(redirectAttributes, "删除案例-造价及材料指标成功");
		return "redirect:"+Global.getAdminPath()+"/casenorm/caseNorm/?repage";
	}
	
	/**
	 * 批量删除案例-造价及材料指标
	 */
	@RequiresPermissions("casenorm:caseNorm:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			caseNormService.delete(caseNormService.get(id));
		}
		addMessage(redirectAttributes, "删除案例-造价及材料指标成功");
		return "redirect:"+Global.getAdminPath()+"/casenorm/caseNorm/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("casenorm:caseNorm:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CaseNorm caseNorm, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-造价及材料指标"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CaseNorm> page = caseNormService.findPage(new Page<CaseNorm>(request, response, -1), caseNorm);
    		new ExportExcel("案例-造价及材料指标", CaseNorm.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出案例-造价及材料指标记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casenorm/caseNorm/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("casenorm:caseNorm:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CaseNorm> list = ei.getDataList(CaseNorm.class);
			for (CaseNorm caseNorm : list){
				try{
					caseNormService.save(caseNorm);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条案例-造价及材料指标记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条案例-造价及材料指标记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入案例-造价及材料指标失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casenorm/caseNorm/?repage";
    }
	
	/**
	 * 下载导入案例-造价及材料指标数据模板
	 */
	@RequiresPermissions("casenorm:caseNorm:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-造价及材料指标数据导入模板.xlsx";
    		List<CaseNorm> list = Lists.newArrayList(); 
    		new ExportExcel("案例-造价及材料指标数据", CaseNorm.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casenorm/caseNorm/?repage";
    }
	
	
	

}