/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.exampleeconomics.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.exampleeconomics.entity.ExampleEconomics;
import com.jeeplus.modules.exampleeconomics.service.ExampleEconomicsService;

/**
 * 经济指标Controller
 * @author ssrh
 * @version 2018-08-29
 */
@Controller
@RequestMapping(value = "${adminPath}/exampleeconomics/exampleEconomics")
public class ExampleEconomicsController extends BaseController {

	@Autowired
	private ExampleEconomicsService exampleEconomicsService;
	
	@ModelAttribute
	public ExampleEconomics get(@RequestParam(required=false) String id) {
		ExampleEconomics entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = exampleEconomicsService.get(id);
		}
		if (entity == null){
			entity = new ExampleEconomics();
		}
		return entity;
	}
	
	/**
	 * 经济指标列表页面
	 */
	@RequiresPermissions("exampleeconomics:exampleEconomics:list")
	@RequestMapping(value = {"list", ""})
	public String list(ExampleEconomics exampleEconomics, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ExampleEconomics> page = exampleEconomicsService.findPage(new Page<ExampleEconomics>(request, response), exampleEconomics); 
		model.addAttribute("page", page);
		return "modules/exampleeconomics/exampleEconomicsList";
	}

	/**
	 * 查看，增加，编辑经济指标表单页面
	 */
	@RequiresPermissions(value={"exampleeconomics:exampleEconomics:view","exampleeconomics:exampleEconomics:add","exampleeconomics:exampleEconomics:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ExampleEconomics exampleEconomics, Model model) {
		model.addAttribute("exampleEconomics", exampleEconomics);
		return "modules/exampleeconomics/exampleEconomicsForm";
	}

	/**
	 * 保存经济指标
	 */
	@RequiresPermissions(value={"exampleeconomics:exampleEconomics:add","exampleeconomics:exampleEconomics:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ExampleEconomics exampleEconomics, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, exampleEconomics)){
			return form(exampleEconomics, model);
		}
		if(!exampleEconomics.getIsNewRecord()){//编辑表单保存
			ExampleEconomics t = exampleEconomicsService.get(exampleEconomics.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(exampleEconomics, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			exampleEconomicsService.save(t);//保存
		}else{//新增表单保存
			exampleEconomicsService.save(exampleEconomics);//保存
		}
		addMessage(redirectAttributes, "保存经济指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleeconomics/exampleEconomics/?repage";
	}
	
	/**
	 * 删除经济指标
	 */
	@RequiresPermissions("exampleeconomics:exampleEconomics:del")
	@RequestMapping(value = "delete")
	public String delete(ExampleEconomics exampleEconomics, RedirectAttributes redirectAttributes) {
		exampleEconomicsService.delete(exampleEconomics);
		addMessage(redirectAttributes, "删除经济指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleeconomics/exampleEconomics/?repage";
	}
	
	/**
	 * 批量删除经济指标
	 */
	@RequiresPermissions("exampleeconomics:exampleEconomics:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			exampleEconomicsService.delete(exampleEconomicsService.get(id));
		}
		addMessage(redirectAttributes, "删除经济指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleeconomics/exampleEconomics/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("exampleeconomics:exampleEconomics:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ExampleEconomics exampleEconomics, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "经济指标"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ExampleEconomics> page = exampleEconomicsService.findPage(new Page<ExampleEconomics>(request, response, -1), exampleEconomics);
    		new ExportExcel("经济指标", ExampleEconomics.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出经济指标记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleeconomics/exampleEconomics/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("exampleeconomics:exampleEconomics:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ExampleEconomics> list = ei.getDataList(ExampleEconomics.class);
			for (ExampleEconomics exampleEconomics : list){
				try{
					exampleEconomicsService.save(exampleEconomics);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条经济指标记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条经济指标记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入经济指标失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleeconomics/exampleEconomics/?repage";
    }
	
	/**
	 * 下载导入经济指标数据模板
	 */
	@RequiresPermissions("exampleeconomics:exampleEconomics:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "经济指标数据导入模板.xlsx";
    		List<ExampleEconomics> list = Lists.newArrayList(); 
    		new ExportExcel("经济指标数据", ExampleEconomics.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleeconomics/exampleEconomics/?repage";
    }
	
	
	

}