/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.exampleproject.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.exampleproject.entity.ExampleProject;
import com.jeeplus.modules.exampleproject.service.ExampleProjectService;

/**
 * 主要工程量指标Controller
 * @author ssrh
 * @version 2018-08-29
 */
@Controller
@RequestMapping(value = "${adminPath}/exampleproject/exampleProject")
public class ExampleProjectController extends BaseController {

	@Autowired
	private ExampleProjectService exampleProjectService;
	
	@ModelAttribute
	public ExampleProject get(@RequestParam(required=false) String id) {
		ExampleProject entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = exampleProjectService.get(id);
		}
		if (entity == null){
			entity = new ExampleProject();
		}
		return entity;
	}
	
	/**
	 * 主要工程量指标列表页面
	 */
	@RequiresPermissions("exampleproject:exampleProject:list")
	@RequestMapping(value = {"list", ""})
	public String list(ExampleProject exampleProject, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ExampleProject> page = exampleProjectService.findPage(new Page<ExampleProject>(request, response), exampleProject); 
		model.addAttribute("page", page);
		return "modules/exampleproject/exampleProjectList";
	}

	/**
	 * 查看，增加，编辑主要工程量指标表单页面
	 */
	@RequiresPermissions(value={"exampleproject:exampleProject:view","exampleproject:exampleProject:add","exampleproject:exampleProject:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ExampleProject exampleProject, Model model) {
		model.addAttribute("exampleProject", exampleProject);
		return "modules/exampleproject/exampleProjectForm";
	}

	/**
	 * 保存主要工程量指标
	 */
	@RequiresPermissions(value={"exampleproject:exampleProject:add","exampleproject:exampleProject:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ExampleProject exampleProject, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, exampleProject)){
			return form(exampleProject, model);
		}
		if(!exampleProject.getIsNewRecord()){//编辑表单保存
			ExampleProject t = exampleProjectService.get(exampleProject.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(exampleProject, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			exampleProjectService.save(t);//保存
		}else{//新增表单保存
			exampleProjectService.save(exampleProject);//保存
		}
		addMessage(redirectAttributes, "保存主要工程量指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleproject/exampleProject/?repage";
	}
	
	/**
	 * 删除主要工程量指标
	 */
	@RequiresPermissions("exampleproject:exampleProject:del")
	@RequestMapping(value = "delete")
	public String delete(ExampleProject exampleProject, RedirectAttributes redirectAttributes) {
		exampleProjectService.delete(exampleProject);
		addMessage(redirectAttributes, "删除主要工程量指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleproject/exampleProject/?repage";
	}
	
	/**
	 * 批量删除主要工程量指标
	 */
	@RequiresPermissions("exampleproject:exampleProject:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			exampleProjectService.delete(exampleProjectService.get(id));
		}
		addMessage(redirectAttributes, "删除主要工程量指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleproject/exampleProject/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("exampleproject:exampleProject:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ExampleProject exampleProject, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "主要工程量指标"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ExampleProject> page = exampleProjectService.findPage(new Page<ExampleProject>(request, response, -1), exampleProject);
    		new ExportExcel("主要工程量指标", ExampleProject.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出主要工程量指标记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleproject/exampleProject/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("exampleproject:exampleProject:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ExampleProject> list = ei.getDataList(ExampleProject.class);
			for (ExampleProject exampleProject : list){
				try{
					exampleProjectService.save(exampleProject);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条主要工程量指标记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条主要工程量指标记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入主要工程量指标失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleproject/exampleProject/?repage";
    }
	
	/**
	 * 下载导入主要工程量指标数据模板
	 */
	@RequiresPermissions("exampleproject:exampleProject:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "主要工程量指标数据导入模板.xlsx";
    		List<ExampleProject> list = Lists.newArrayList(); 
    		new ExportExcel("主要工程量指标数据", ExampleProject.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleproject/exampleProject/?repage";
    }
	
	
	

}