/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sysparameter.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.sysparameter.entity.SysParameterValue;
import com.jeeplus.modules.sysparameter.service.SysParameterValueService;

/**
 * 系统参数值Controller
 * @author chenyuping
 * @version 2018-08-13
 */
@Controller
@RequestMapping(value = "${adminPath}/sysparameter/sysParameterValue")
public class SysParameterValueController extends BaseController {

	@Autowired
	private SysParameterValueService sysParameterValueService;
	
	@ModelAttribute
	public SysParameterValue get(@RequestParam(required=false) String id) {
		SysParameterValue entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysParameterValueService.get(id);
		}
		if (entity == null){
			entity = new SysParameterValue();
		}
		return entity;
	}
	
	/**
	 * 系统参数值列表页面
	 */
	@RequiresPermissions("sysparameter:sysParameterValue:list")
	@RequestMapping(value = {"list", ""})
	public String list(SysParameterValue sysParameterValue, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SysParameterValue> page = sysParameterValueService.findPage(new Page<SysParameterValue>(request, response), sysParameterValue); 
		model.addAttribute("page", page);
		return "modules/sysparameter/sysParameterValueList";
	}

	/**
	 * 查看，增加，编辑系统参数值表单页面
	 */
	@RequiresPermissions(value={"sysparameter:sysParameterValue:view","sysparameter:sysParameterValue:add","sysparameter:sysParameterValue:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysParameterValue sysParameterValue, Model model) {
		model.addAttribute("sysParameterValue", sysParameterValue);
		return "modules/sysparameter/sysParameterValueForm";
	}

	/**
	 * 保存系统参数值
	 */
	@RequiresPermissions(value={"sysparameter:sysParameterValue:add","sysparameter:sysParameterValue:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysParameterValue sysParameterValue, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysParameterValue)){
			return form(sysParameterValue, model);
		}
		if(!sysParameterValue.getIsNewRecord()){//编辑表单保存
			SysParameterValue t = sysParameterValueService.get(sysParameterValue.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(sysParameterValue, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			sysParameterValueService.save(t);//保存
		}else{//新增表单保存
			sysParameterValueService.save(sysParameterValue);//保存
		}
		addMessage(redirectAttributes, "保存系统参数值成功");
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameterValue/?repage";
	}
	
	/**
	 * 删除系统参数值
	 */
	@RequiresPermissions("sysparameter:sysParameterValue:del")
	@RequestMapping(value = "delete")
	public String delete(SysParameterValue sysParameterValue, RedirectAttributes redirectAttributes) {
		sysParameterValueService.delete(sysParameterValue);
		addMessage(redirectAttributes, "删除系统参数值成功");
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameterValue/?repage";
	}
	
	/**
	 * 批量删除系统参数值
	 */
	@RequiresPermissions("sysparameter:sysParameterValue:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysParameterValueService.delete(sysParameterValueService.get(id));
		}
		addMessage(redirectAttributes, "删除系统参数值成功");
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameterValue/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("sysparameter:sysParameterValue:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysParameterValue sysParameterValue, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "系统参数值"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysParameterValue> page = sysParameterValueService.findPage(new Page<SysParameterValue>(request, response, -1), sysParameterValue);
    		new ExportExcel("系统参数值", SysParameterValue.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出系统参数值记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameterValue/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("sysparameter:sysParameterValue:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysParameterValue> list = ei.getDataList(SysParameterValue.class);
			for (SysParameterValue sysParameterValue : list){
				try{
					sysParameterValueService.save(sysParameterValue);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条系统参数值记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条系统参数值记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入系统参数值失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameterValue/?repage";
    }
	
	/**
	 * 下载导入系统参数值数据模板
	 */
	@RequiresPermissions("sysparameter:sysParameterValue:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "系统参数值数据导入模板.xlsx";
    		List<SysParameterValue> list = Lists.newArrayList(); 
    		new ExportExcel("系统参数值数据", SysParameterValue.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysparameter/sysParameterValue/?repage";
    }
	
	
	

}