/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.syswarning.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.syswarning.entity.SysWarningUser;
import com.jeeplus.modules.syswarning.service.SysWarningUserService;

/**
 * 系统预警Controller
 * @author chenyuping
 * @version 2018-08-14
 */
@Controller
@RequestMapping(value = "${adminPath}/syswarning/sysWarningUser")
public class SysWarningUserController extends BaseController {

	@Autowired
	private SysWarningUserService sysWarningUserService;
	
	@ModelAttribute
	public SysWarningUser get(@RequestParam(required=false) String id) {
		SysWarningUser entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysWarningUserService.get(id);
		}
		if (entity == null){
			entity = new SysWarningUser();
		}
		return entity;
	}
	
	/**
	 * 系统预警列表页面
	 */
	@RequiresPermissions("syswarning:sysWarningUser:list")
	@RequestMapping(value = {"list", ""})
	public String list(SysWarningUser sysWarningUser, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SysWarningUser> page = sysWarningUserService.findPage(new Page<SysWarningUser>(request, response), sysWarningUser); 
		model.addAttribute("page", page);
		return "modules/syswarning/sysWarningUserList";
	}

	/**
	 * 查看，增加，编辑系统预警表单页面
	 */
	@RequiresPermissions(value={"syswarning:sysWarningUser:view","syswarning:sysWarningUser:add","syswarning:sysWarningUser:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysWarningUser sysWarningUser, Model model) {
		model.addAttribute("sysWarningUser", sysWarningUser);
		return "modules/syswarning/sysWarningUserForm";
	}

	/**
	 * 保存系统预警
	 */
	@RequiresPermissions(value={"syswarning:sysWarningUser:add","syswarning:sysWarningUser:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysWarningUser sysWarningUser, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysWarningUser)){
			return form(sysWarningUser, model);
		}
		if(!sysWarningUser.getIsNewRecord()){//编辑表单保存
			SysWarningUser t = sysWarningUserService.get(sysWarningUser.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(sysWarningUser, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			sysWarningUserService.save(t);//保存
		}else{//新增表单保存
			sysWarningUserService.save(sysWarningUser);//保存
		}
		addMessage(redirectAttributes, "保存系统预警成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningUser/?repage";
	}
	
	/**
	 * 删除系统预警
	 */
	@RequiresPermissions("syswarning:sysWarningUser:del")
	@RequestMapping(value = "delete")
	public String delete(SysWarningUser sysWarningUser, RedirectAttributes redirectAttributes) {
		sysWarningUserService.delete(sysWarningUser);
		addMessage(redirectAttributes, "删除系统预警成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningUser/?repage";
	}
	
	/**
	 * 批量删除系统预警
	 */
	@RequiresPermissions("syswarning:sysWarningUser:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysWarningUserService.delete(sysWarningUserService.get(id));
		}
		addMessage(redirectAttributes, "删除系统预警成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningUser/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("syswarning:sysWarningUser:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysWarningUser sysWarningUser, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "系统预警"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysWarningUser> page = sysWarningUserService.findPage(new Page<SysWarningUser>(request, response, -1), sysWarningUser);
    		new ExportExcel("系统预警", SysWarningUser.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出系统预警记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningUser/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("syswarning:sysWarningUser:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysWarningUser> list = ei.getDataList(SysWarningUser.class);
			for (SysWarningUser sysWarningUser : list){
				try{
					sysWarningUserService.save(sysWarningUser);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条系统预警记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条系统预警记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入系统预警失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningUser/?repage";
    }
	
	/**
	 * 下载导入系统预警数据模板
	 */
	@RequiresPermissions("syswarning:sysWarningUser:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "系统预警数据导入模板.xlsx";
    		List<SysWarningUser> list = Lists.newArrayList(); 
    		new ExportExcel("系统预警数据", SysWarningUser.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningUser/?repage";
    }
	
	
	

}