/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workattendance.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.entity.Office;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workattendance.entity.WorkAttendanceMonth;
import com.jeeplus.modules.workattendance.service.WorkAttendanceMonthService;

/**
 * 考勤月统计Controller
 * @author 杨帆
 * @version 2018-09-10
 */
@Controller
@RequestMapping(value = "${adminPath}/workattendance/workAttendanceMonth")
public class WorkAttendanceMonthController extends BaseController {

	@Autowired
	private WorkAttendanceMonthService workAttendanceMonthService;
	
	@ModelAttribute
	public WorkAttendanceMonth get(@RequestParam(required=false) String id) {
		WorkAttendanceMonth entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workAttendanceMonthService.get(id);
		}
		if (entity == null){
			entity = new WorkAttendanceMonth();
		}
		return entity;
	}
	
	/**
	 * 考勤月统计列表页面
	 */
	@RequiresPermissions("workattendance:workAttendanceMonth:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkAttendanceMonth workAttendanceMonth, HttpServletRequest request, HttpServletResponse response, Model model) {
		String branchOffice = UserUtils.getSelectBranchOffice();
		workAttendanceMonth.setBranchOffice(branchOffice);
		List<WorkAttendanceMonth> list = workAttendanceMonthService.findList(workAttendanceMonth);
		model.addAttribute("list", list);
		return "modules/workattendance/workAttendanceMonthList";
	}

	/**
	 * 查看，增加，编辑考勤月统计表单页面
	 */
	@RequiresPermissions(value={"workattendance:workAttendanceMonth:view","workattendance:workAttendanceMonth:add","workattendance:workAttendanceMonth:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkAttendanceMonth workAttendanceMonth, Model model) {
		model.addAttribute("workAttendanceMonth", workAttendanceMonth);
		return "modules/workattendance/workAttendanceMonthForm";
	}

	/**
	 * 保存考勤月统计
	 */
	@RequiresPermissions(value={"workattendance:workAttendanceMonth:add","workattendance:workAttendanceMonth:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkAttendanceMonth workAttendanceMonth, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workAttendanceMonth)){
			return form(workAttendanceMonth, model);
		}
		if(!workAttendanceMonth.getIsNewRecord()){//编辑表单保存
			WorkAttendanceMonth t = workAttendanceMonthService.get(workAttendanceMonth.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workAttendanceMonth, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workAttendanceMonthService.save(t);//保存
		}else{//新增表单保存
			workAttendanceMonthService.save(workAttendanceMonth);//保存
		}
		addMessage(redirectAttributes, "保存考勤月统计成功");
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceMonth/?repage";
	}
	
	/**
	 * 删除考勤月统计
	 */
	@RequiresPermissions("workattendance:workAttendanceMonth:del")
	@RequestMapping(value = "delete")
	public String delete(WorkAttendanceMonth workAttendanceMonth, RedirectAttributes redirectAttributes) {
		workAttendanceMonthService.delete(workAttendanceMonth);
		addMessage(redirectAttributes, "删除考勤月统计成功");
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceMonth/?repage";
	}
	
	/**
	 * 批量删除考勤月统计
	 */
	@RequiresPermissions("workattendance:workAttendanceMonth:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workAttendanceMonthService.delete(workAttendanceMonthService.get(id));
		}
		addMessage(redirectAttributes, "删除考勤月统计成功");
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceMonth/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workattendance:workAttendanceMonth:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkAttendanceMonth workAttendanceMonth, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "考勤月统计"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkAttendanceMonth> page = workAttendanceMonthService.findPage(new Page<WorkAttendanceMonth>(request, response, -1), workAttendanceMonth);
    		new ExportExcel("考勤月统计", WorkAttendanceMonth.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出考勤月统计记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceMonth/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workattendance:workAttendanceMonth:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkAttendanceMonth> list = ei.getDataList(WorkAttendanceMonth.class);
			for (WorkAttendanceMonth workAttendanceMonth : list){
				try{
					workAttendanceMonthService.save(workAttendanceMonth);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条考勤月统计记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条考勤月统计记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入考勤月统计失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceMonth/?repage";
    }
	
	/**
	 * 下载导入考勤月统计数据模板
	 */
	@RequiresPermissions("workattendance:workAttendanceMonth:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "考勤月统计数据导入模板.xlsx";
    		List<WorkAttendanceMonth> list = Lists.newArrayList(); 
    		new ExportExcel("考勤月统计数据", WorkAttendanceMonth.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceMonth/?repage";
    }
	
	
	

}