/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentPricemanage;
import com.jeeplus.modules.workcontent.service.WorkContentPricemanageService;

/**
 * 认价管理Controller
 * @author ssrh
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentPricemanage")
public class WorkContentPricemanageController extends BaseController {

	@Autowired
	private WorkContentPricemanageService workContentPricemanageService;
	
	@ModelAttribute
	public WorkContentPricemanage get(@RequestParam(required=false) String id) {
		WorkContentPricemanage entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentPricemanageService.get(id);
		}
		if (entity == null){
			entity = new WorkContentPricemanage();
		}
		return entity;
	}
	
	/**
	 * 认价管理列表页面
	 */
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentPricemanage workContentPricemanage, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentPricemanage> page = workContentPricemanageService.findPage(new Page<WorkContentPricemanage>(request, response), workContentPricemanage); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentPricemanageList";
	}

	/**
	 * 查看，增加，编辑认价管理表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentPricemanage workContentPricemanage,String view, Model model) {
        WorkContentPricemanage oldRecord = null;
        if(StringUtils.isNotBlank(workContentPricemanage.getContentId())){
            List<WorkContentPricemanage> list = workContentPricemanageService.findList(workContentPricemanage);
            if(list!=null&&!list.isEmpty()){
                oldRecord = list.get(0);
            }
        }
        if(oldRecord!=null){
            model.addAttribute("workContentPricemanage", oldRecord);
        }else {
            model.addAttribute("workContentPricemanage", workContentPricemanage);
        }
        String viewPath = "modules/workcontent/workContentPricemanageForm";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存认价管理
	 */
	@RequestMapping(value = "save")
	public String save(WorkContentPricemanage workContentPricemanage, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentPricemanage)){
			return form(workContentPricemanage, null,model);
		}
		if(!workContentPricemanage.getIsNewRecord()){//编辑表单保存
			WorkContentPricemanage t = workContentPricemanageService.get(workContentPricemanage.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentPricemanage, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentPricemanageService.save(t);//保存
		}else{//新增表单保存
			workContentPricemanageService.save(workContentPricemanage);//保存
		}
		addMessage(redirectAttributes, "保存认价管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPricemanage/?repage";
	}
	
	/**
	 * 删除认价管理
	 */
	@RequiresPermissions("workcontent:workContentPricemanage:del")
	@RequestMapping(value = "delete")
	public String delete(WorkContentPricemanage workContentPricemanage, RedirectAttributes redirectAttributes) {
		workContentPricemanageService.delete(workContentPricemanage);
		addMessage(redirectAttributes, "删除认价管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPricemanage/?repage";
	}
	
	/**
	 * 批量删除认价管理
	 */
	@RequiresPermissions("workcontent:workContentPricemanage:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentPricemanageService.delete(workContentPricemanageService.get(id));
		}
		addMessage(redirectAttributes, "删除认价管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPricemanage/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontent:workContentPricemanage:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentPricemanage workContentPricemanage, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "认价管理"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentPricemanage> page = workContentPricemanageService.findPage(new Page<WorkContentPricemanage>(request, response, -1), workContentPricemanage);
    		new ExportExcel("认价管理", WorkContentPricemanage.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出认价管理记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPricemanage/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontent:workContentPricemanage:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentPricemanage> list = ei.getDataList(WorkContentPricemanage.class);
			for (WorkContentPricemanage workContentPricemanage : list){
				try{
					workContentPricemanageService.save(workContentPricemanage);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条认价管理记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条认价管理记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入认价管理失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPricemanage/?repage";
    }
	
	/**
	 * 下载导入认价管理数据模板
	 */
	@RequiresPermissions("workcontent:workContentPricemanage:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "认价管理数据导入模板.xlsx";
    		List<WorkContentPricemanage> list = Lists.newArrayList(); 
    		new ExportExcel("认价管理数据", WorkContentPricemanage.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentPricemanage/?repage";
    }
	
	
	

}