/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontractinfo.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.common.persistence.Page;
import com.jeeplus.modules.workcontractinfo.entity.WorkContractReview;
import com.jeeplus.modules.workcontractinfo.service.WorkContractReviewService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;

/**
 * 合同评价表Controller
 * @author 杨帆
 * @version 2018-03-23
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontractreview/workcontractreview")
public class WorkcontractreviewController extends BaseController {
	@Autowired
	private WorkContractReviewService workContractReviewService;
	
	@ModelAttribute
	public WorkContractReview get(@RequestParam(required=false) String id) {
		WorkContractReview entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContractReviewService.get(id);
		}
		if (entity == null){
			entity = new WorkContractReview();
		}
		return entity;
	}
	
	/**
	 * 合同评价信息列表页面
	 */
	@RequiresPermissions("workcontractreview:workcontractreview:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkContractReview workContractReview, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContractReview> page = workContractReviewService.findPage(new Page<WorkContractReview>(request, response), workContractReview);
		model.addAttribute("page", page);
		return "workcontractreview/workcontractreviewList";
	}

	/**
	 * 查看，增加，编辑合同评价信息表单页面
	 */
	@RequiresPermissions(value={"workcontractreview:workcontractreview:view","workcontractreview:workcontractreview:add","workcontractreview:workcontractreview:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkContractReview workContractReview, Model model) {
		model.addAttribute("workcontractreview", workContractReview);
		return "workcontractreview/workcontractreviewForm";
	}

	/**
	 * 保存合同评价信息
	 */
	@RequiresPermissions(value={"workcontractreview:workcontractreview:add","workcontractreview:workcontractreview:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkContractReview workContractReview, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContractReview)){
			return form(workContractReview, model);
		}
		if(!workContractReview.getIsNewRecord()){//编辑表单保存
			WorkContractReview t = workContractReviewService.get(workContractReview.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContractReview, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContractReviewService.save(t);//保存
		}else{//新增表单保存
			workContractReviewService.save(workContractReview);//保存
		}
		addMessage(redirectAttributes, "保存合同评价信息成功");
		return "redirect:"+Global.getAdminPath()+"/workcontractreview/workcontractreview/?repage";
	}
	
	/**
	 * 删除合同评价信息
	 */
	@RequiresPermissions("workcontractreview:workcontractreview:del")
	@RequestMapping(value = "delete")
	public String delete(WorkContractReview workContractReview, RedirectAttributes redirectAttributes) {
		workContractReviewService.delete(workContractReview);
		addMessage(redirectAttributes, "删除合同评价信息成功");
		return "redirect:"+Global.getAdminPath()+"/workcontractreview/workcontractreview/?repage";
	}
	
	/**
	 * 批量删除合同评价信息
	 */
	@RequiresPermissions("workcontractreview:workcontractreview:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContractReviewService.delete(workContractReviewService.get(id));
		}
		addMessage(redirectAttributes, "删除合同评价信息成功");
		return "redirect:"+Global.getAdminPath()+"/workcontractreview/workcontractreview/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontractreview:workcontractreview:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContractReview workContractReview, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "合同评价信息"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContractReview> page = workContractReviewService.findPage(new Page<WorkContractReview>(request, response, -1), workContractReview);
			String agent = request.getHeader("USER-AGENT");
            new ExportExcel("合同评价信息", WorkContractReview.class).setDataList(page.getList()).write(response, fileName,agent).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出合同评价信息记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontractreview/workcontractreview/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontractreview:workcontractreview:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContractReview> list = ei.getDataList(WorkContractReview.class);
			for (WorkContractReview workContractReview : list){
				try{
					workContractReviewService.save(workContractReview);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条合同评价信息记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条合同评价信息记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入合同评价信息失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontractreview/workcontractreview/?repage";
    }
	
	/**
	 * 下载导入合同评价信息数据模板
	 */
	@RequiresPermissions("workcontractreview:workcontractreview:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletRequest request,HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "合同评价信息数据导入模板.xlsx";
    		List<WorkContractReview> list = Lists.newArrayList();
			String agent = request.getHeader("USER-AGENT");
    		new ExportExcel("合同评价信息数据", WorkContractReview.class, 1).setDataList(list).write(response, fileName,agent).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontractreview/workcontractreview/?repage";
    }
	
	
	

}