/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workhandbill.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workhandbill.entity.WorkHandBillDetail;
import com.jeeplus.modules.workhandbill.service.WorkHandBillDetailService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 交接单材料Controller
 * @author 马鹏波
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/workhandbill/workHandBillDetail")
public class WorkHandBillDetailController extends BaseController {

	@Autowired
	private WorkHandBillDetailService workHandBillDetailService;
	
	@ModelAttribute
	public WorkHandBillDetail get(@RequestParam(required=false) String id) {
		WorkHandBillDetail entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workHandBillDetailService.get(id);
		}
		if (entity == null){
			entity = new WorkHandBillDetail();
		}
		return entity;
	}
	
	/**
	 * 交接单材料列表页面
	 */
//	@RequiresPermissions("workhandbill:workHandBillDetail:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkHandBillDetail workHandBillDetail, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkHandBillDetail> page = workHandBillDetailService.findPage(new Page<WorkHandBillDetail>(request, response), workHandBillDetail); 
		model.addAttribute("page", page);
		return "modules/workhandbill/workHandBillDetailList";
	}

	/**
	 * 查看，增加，编辑交接单材料表单页面
	 */
//	@RequiresPermissions(value={"workhandbill:workHandBillDetail:view","workhandbill:workHandBillDetail:add","workhandbill:workHandBillDetail:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkHandBillDetail workHandBillDetail, Model model) {
		model.addAttribute("workHandBillDetail", workHandBillDetail);
		return "modules/workhandbill/workHandBillDetailForm";
	}

	/**
	 * 保存交接单材料
	 */
//	@RequiresPermissions(value={"workhandbill:workHandBillDetail:add","workhandbill:workHandBillDetail:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkHandBillDetail workHandBillDetail, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workHandBillDetail)){
			return form(workHandBillDetail, model);
		}
		if(!workHandBillDetail.getIsNewRecord()){//编辑表单保存
			WorkHandBillDetail t = workHandBillDetailService.get(workHandBillDetail.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workHandBillDetail, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workHandBillDetailService.save(t);//保存
		}else{//新增表单保存
			workHandBillDetailService.save(workHandBillDetail);//保存
		}
		addMessage(redirectAttributes, "保存交接单材料成功");
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBillDetail/?repage";
	}
	
	/**
	 * 删除交接单材料
	 */
//	@RequiresPermissions("workhandbill:workHandBillDetail:del")
	@RequestMapping(value = "delete")
	public String delete(WorkHandBillDetail workHandBillDetail, RedirectAttributes redirectAttributes) {
		workHandBillDetailService.delete(workHandBillDetail);
		addMessage(redirectAttributes, "删除交接单材料成功");
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBillDetail/?repage";
	}
	
	/**
	 * 批量删除交接单材料
	 */
//	@RequiresPermissions("workhandbill:workHandBillDetail:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workHandBillDetailService.delete(workHandBillDetailService.get(id));
		}
		addMessage(redirectAttributes, "删除交接单材料成功");
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBillDetail/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workhandbill:workHandBillDetail:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkHandBillDetail workHandBillDetail, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "交接单材料"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkHandBillDetail> page = workHandBillDetailService.findPage(new Page<WorkHandBillDetail>(request, response, -1), workHandBillDetail);
    		new ExportExcel("交接单材料", WorkHandBillDetail.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出交接单材料记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBillDetail/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workhandbill:workHandBillDetail:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkHandBillDetail> list = ei.getDataList(WorkHandBillDetail.class);
			for (WorkHandBillDetail workHandBillDetail : list){
				try{
					workHandBillDetailService.save(workHandBillDetail);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条交接单材料记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条交接单材料记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入交接单材料失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBillDetail/?repage";
    }
	
	/**
	 * 下载导入交接单材料数据模板
	 */
//	@RequiresPermissions("workhandbill:workHandBillDetail:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "交接单材料数据导入模板.xlsx";
    		List<WorkHandBillDetail> list = Lists.newArrayList(); 
    		new ExportExcel("交接单材料数据", WorkHandBillDetail.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBillDetail/?repage";
    }
	
	
	

}