/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.worklog.web;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.HttpClientUtil;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.worklog.entity.WorkLog;
import com.jeeplus.modules.worklog.service.WorkLogService;
import net.sf.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Controller
@RequestMapping(value = "${adminPath}/worklog/workLog")
public class WorkLogController extends BaseController {

	@Autowired
	private WorkLogService workLogService;
	
	@ModelAttribute
	public WorkLog get(@RequestParam(required=false) String id) {
		WorkLog entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workLogService.get(id);
		}
		if (entity == null){
			entity = new WorkLog();
		}
		return entity;
	}
	
	/**
	 * 日志列表页面
	 */
//	@RequiresPermissions("worklog:worklog:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkLog workLog, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkLog> page = workLogService.findPage(new Page<WorkLog>(request, response), workLog); 
		model.addAttribute("page", page);
		return "modules/worklog/workLogList";
	}

	/**
	 * 查看，增加，编辑表单页面
	 */
//	@RequiresPermissions(value={"worklog:worklog:view","worklog:worklog:add","worklog:worklog:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkLog workLog,HttpServletRequest request, Model model) {
		String view = request.getParameter("view");
		if("view".equals(view)){
			view = "modules/worklog/workLogFormView";
		}else{
			view = "modules/worklog/workLogForm";
		}
		if(!Strings.isNullOrEmpty(workLog.getProjectId())){
			workLog = workLogService.get(workLog.getProjectId());
		}
		model.addAttribute("workLog", workLog);
		return view;
	}

	/**
	 * 保存记录
	 */
//	@RequiresPermissions(value={"worklog:worklog:add","worklog:worklog:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkLog workLog, HttpServletRequest request,Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workLog)){
			return form(workLog,request, model);
		}
		if(!workLog.getIsNewRecord()){//编辑表单保存
			WorkLog t = workLogService.get(workLog.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workLog, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workLogService.save(t);//保存
		}else{//新增表单保存
			workLogService.save(workLog);//保存
		}
		addMessage(redirectAttributes, "保存记录成功");
		return "redirect:"+Global.getAdminPath()+"/worklog/worklog/?repage";
	}
	

//	@RequiresPermissions("worklog:worklog:del")
	@RequestMapping(value = "delete")
	public String delete(WorkLog workLog, RedirectAttributes redirectAttributes) {
		workLogService.delete(workLog);
		addMessage(redirectAttributes, "删除日志成功");
		return "redirect:"+Global.getAdminPath()+"/worklog/worklog/?repage";
	}
	

//	@RequiresPermissions("worklog:worklog:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workLogService.delete(workLogService.get(id));
		}
		addMessage(redirectAttributes, "删除日志成功");
		return "redirect:"+Global.getAdminPath()+"/worklog/worklog/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("worklog:worklog:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkLog workLog, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "日志"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkLog> page = workLogService.findPage(new Page<WorkLog>(request, response, -1), workLog);
    		new ExportExcel("日志", WorkLog.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出日志失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/worklog/worklog/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("worklog:worklog:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkLog> list = ei.getDataList(WorkLog.class);
			for (WorkLog workLog : list){
				try{
					workLogService.save(workLog);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/worklog/worklog/?repage";
    }
	
	/**
	 * 下载导入现场踏勘记录数据模板
	 */
//	@RequiresPermissions("worklog:worklog:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "日志导入模板.xlsx";
    		List<WorkLog> list = Lists.newArrayList(); 
    		new ExportExcel("日志", WorkLog.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/worklog/worklog/?repage";
    }

    @ResponseBody
	@RequestMapping(value = "getWeather")
	public Map getWeather(String code){
    	Map map = new HashMap();
    	try {
			String url = "http://t.weather.sojson.com/api/weather/city/" + code;
			JSONObject jsonObject = HttpClientUtil.httpRequest(url, "GET", null);
			map.put("code", 1);
			map.put("data", jsonObject);
		}catch (Exception e){
    		map.put("code",0);
		}

		return map;
	}
	
	
	

}