/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.worklog.web;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.worklog.entity.WorkMeeting;
import com.jeeplus.modules.worklog.service.WorkLogService;
import com.jeeplus.modules.worklog.service.WorkMeetingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;


@Controller
@RequestMapping(value = "${adminPath}/workmeeting/workMeeting")
public class WorkMeetingController extends BaseController {

	@Autowired
	private WorkMeetingService workMeetingService;
	
	@ModelAttribute
	public WorkMeeting get(@RequestParam(required=false) String id) {
		WorkMeeting entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workMeetingService.get(id);
		}
		if (entity == null){
			entity = new WorkMeeting();
		}
		return entity;
	}
	
	/**
	 * 日志列表页面
	 */
//	@RequiresPermissions("workMeeting:workMeeting:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkMeeting workMeeting, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkMeeting> page = workMeetingService.findPage(new Page<WorkMeeting>(request, response), workMeeting); 
		model.addAttribute("page", page);
		return "modules/workMeeting/workMeetingList";
	}

	/**
	 * 查看，增加，编辑表单页面
	 */
//	@RequiresPermissions(value={"workMeeting:workMeeting:view","workMeeting:workMeeting:add","workMeeting:workMeeting:edit"},Meetingical=Meetingical.OR)
	@RequestMapping(value = "form")
	public String form(WorkMeeting workMeeting,HttpServletRequest request, Model model) {
		String view = request.getParameter("view");
		if("view".equals(view)){
			view = "modules/worklog/workMeetFormView";
		}else{
			view = "modules/worklog/workMeetingForm";
		}
		if(!Strings.isNullOrEmpty(workMeeting.getProjectId())){
			workMeeting = workMeetingService.get(workMeeting.getProjectId());
		}
		model.addAttribute("workMeeting", workMeeting);
		return view;
	}

	/**
	 * 保存记录
	 */
//	@RequiresPermissions(value={"workMeeting:workMeeting:add","workMeeting:workMeeting:edit"},Meetingical=Meetingical.OR)
	@RequestMapping(value = "save")
	public String save(WorkMeeting workMeeting, HttpServletRequest request,Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workMeeting)){
			return form(workMeeting,request, model);
		}
		if(!workMeeting.getIsNewRecord()){//编辑表单保存
			WorkMeeting t = workMeetingService.get(workMeeting.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workMeeting, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workMeetingService.save(t);//保存
		}else{//新增表单保存
			workMeetingService.save(workMeeting);//保存
		}
		addMessage(redirectAttributes, "保存记录成功");
		return "redirect:"+Global.getAdminPath()+"/workMeeting/workMeeting/?repage";
	}
	

//	@RequiresPermissions("workMeeting:workMeeting:del")
	@RequestMapping(value = "delete")
	public String delete(WorkMeeting workMeeting, RedirectAttributes redirectAttributes) {
		workMeetingService.delete(workMeeting);
		addMessage(redirectAttributes, "删除日志成功");
		return "redirect:"+Global.getAdminPath()+"/workMeeting/workMeeting/?repage";
	}
	

//	@RequiresPermissions("workMeeting:workMeeting:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workMeetingService.delete(workMeetingService.get(id));
		}
		addMessage(redirectAttributes, "删除日志成功");
		return "redirect:"+Global.getAdminPath()+"/workMeeting/workMeeting/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workMeeting:workMeeting:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkMeeting workMeeting, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "日志"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkMeeting> page = workMeetingService.findPage(new Page<WorkMeeting>(request, response, -1), workMeeting);
    		new ExportExcel("日志", WorkMeeting.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出日志失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workMeeting/workMeeting/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workMeeting:workMeeting:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkMeeting> list = ei.getDataList(WorkMeeting.class);
			for (WorkMeeting workMeeting : list){
				try{
					workMeetingService.save(workMeeting);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workMeeting/workMeeting/?repage";
    }
	
	/**
	 * 下载导入现场踏勘记录数据模板
	 */
//	@RequiresPermissions("workMeeting:workMeeting:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "日志导入模板.xlsx";
    		List<WorkMeeting> list = Lists.newArrayList(); 
    		new ExportExcel("日志", WorkMeeting.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workMeeting/workMeeting/?repage";
    }
	
	
	

}