/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workmonthlyplan.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workmonthlyplan.entity.WorkMonthlyPlanThis;
import com.jeeplus.modules.workmonthlyplan.service.WorkMonthlyPlanThisService;

/**
 * 本月计划Controller
 * @author cyp
 * @version 2018-06-27
 */
@Controller
@RequestMapping(value = "${adminPath}/workmonthlyplanthis/workMonthlyPlanThis")
public class WorkMonthlyPlanThisController extends BaseController {

	@Autowired
	private WorkMonthlyPlanThisService workMonthlyPlanThisService;
	
	@ModelAttribute
	public WorkMonthlyPlanThis get(@RequestParam(required=false) String id) {
		WorkMonthlyPlanThis entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workMonthlyPlanThisService.get(id);
		}
		if (entity == null){
			entity = new WorkMonthlyPlanThis();
		}
		return entity;
	}
	
	/**
	 * 本月计划列表页面
	 */
	@RequiresPermissions("workmonthlyplanthis:workMonthlyPlanThis:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkMonthlyPlanThis workMonthlyPlanThis, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkMonthlyPlanThis> page = workMonthlyPlanThisService.findPage(new Page<WorkMonthlyPlanThis>(request, response), workMonthlyPlanThis); 
		model.addAttribute("page", page);
		return "modules/workmonthlyplan/workMonthlyPlanThisList";
	}

	/**
	 * 查看，增加，编辑本月计划表单页面
	 */
	@RequiresPermissions(value={"workmonthlyplanthis:workMonthlyPlanThis:view","workmonthlyplanthis:workMonthlyPlanThis:add","workmonthlyplanthis:workMonthlyPlanThis:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkMonthlyPlanThis workMonthlyPlanThis, Model model) {
		model.addAttribute("workMonthlyPlanThis", workMonthlyPlanThis);
		return "modules/workmonthlyplan/workMonthlyPlanThisForm";
	}

	/**
	 * 保存本月计划
	 */
	@RequiresPermissions(value={"workmonthlyplanthis:workMonthlyPlanThis:add","workmonthlyplanthis:workMonthlyPlanThis:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkMonthlyPlanThis workMonthlyPlanThis, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workMonthlyPlanThis)){
			return form(workMonthlyPlanThis, model);
		}
		if(!workMonthlyPlanThis.getIsNewRecord()){//编辑表单保存
			WorkMonthlyPlanThis t = workMonthlyPlanThisService.get(workMonthlyPlanThis.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workMonthlyPlanThis, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workMonthlyPlanThisService.save(t);//保存
		}else{//新增表单保存
			workMonthlyPlanThisService.save(workMonthlyPlanThis);//保存
		}
		addMessage(redirectAttributes, "保存本月计划成功");
		return "redirect:"+Global.getAdminPath()+"/workmonthlyplanthis/workMonthlyPlanThis/?repage";
	}
	
	/**
	 * 删除本月计划
	 */
	@RequiresPermissions("workmonthlyplanthis:workMonthlyPlanThis:del")
	@RequestMapping(value = "delete")
	public String delete(WorkMonthlyPlanThis workMonthlyPlanThis, RedirectAttributes redirectAttributes) {
		workMonthlyPlanThisService.delete(workMonthlyPlanThis);
		addMessage(redirectAttributes, "删除本月计划成功");
		return "redirect:"+Global.getAdminPath()+"/workmonthlyplanthis/workMonthlyPlanThis/?repage";
	}
	
	/**
	 * 批量删除本月计划
	 */
	@RequiresPermissions("workmonthlyplanthis:workMonthlyPlanThis:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workMonthlyPlanThisService.delete(workMonthlyPlanThisService.get(id));
		}
		addMessage(redirectAttributes, "删除本月计划成功");
		return "redirect:"+Global.getAdminPath()+"/workmonthlyplanthis/workMonthlyPlanThis/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workmonthlyplanthis:workMonthlyPlanThis:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkMonthlyPlanThis workMonthlyPlanThis, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "本月计划"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkMonthlyPlanThis> page = workMonthlyPlanThisService.findPage(new Page<WorkMonthlyPlanThis>(request, response, -1), workMonthlyPlanThis);
    		new ExportExcel("本月计划", WorkMonthlyPlanThis.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出本月计划记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmonthlyplanthis/workMonthlyPlanThis/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workmonthlyplanthis:workMonthlyPlanThis:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkMonthlyPlanThis> list = ei.getDataList(WorkMonthlyPlanThis.class);
			for (WorkMonthlyPlanThis workMonthlyPlanThis : list){
				try{
					workMonthlyPlanThisService.save(workMonthlyPlanThis);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条本月计划记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条本月计划记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入本月计划失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmonthlyplanthis/workMonthlyPlanThis/?repage";
    }
	
	/**
	 * 下载导入本月计划数据模板
	 */
	@RequiresPermissions("workmonthlyplanthis:workMonthlyPlanThis:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "本月计划数据导入模板.xlsx";
    		List<WorkMonthlyPlanThis> list = Lists.newArrayList(); 
    		new ExportExcel("本月计划数据", WorkMonthlyPlanThis.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmonthlyplanthis/workMonthlyPlanThis/?repage";
    }
	
	
	

}